package com.ipsplm.kafka;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ipsplm.entity.iot.*;
import com.ipsplm.service.IXaIotDeviceListSyncService;
import com.ipsplm.service.IXaMeterListService;
import com.ipsplm.utils.RedisUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Description kafka消费者监听
 * @Author FanDongqiang
 * @Date 2023/4/26 10:19
 * @Version 1.0
 */
@Component
@Slf4j
public class KafkaConsumerListener {
    private IXaIotDeviceListSyncService iXaIotDeviceListSyncService;
    private IXaMeterListService iXaMeterListService;

    @Autowired
    private RedisUtils redisUtils;

    // 设备编号List
    private static Map<String, Integer> xaDeviceNumMap = new ConcurrentHashMap<>();
    //仪表编号List
    private static Map<String,Integer> xaMeterNumMap = new ConcurrentHashMap<>();

    private final Map<String, Long> deviceTimestamps = new ConcurrentHashMap<>();
    public KafkaConsumerListener(IXaIotDeviceListSyncService iXaIotDeviceListSyncService, IXaMeterListService iXaMeterListService) {
        this.iXaIotDeviceListSyncService = iXaIotDeviceListSyncService;
        this.iXaMeterListService = iXaMeterListService;
        //设备IOT清单
        List<XaIotDeviceListSync> xaDeviceList = this.iXaIotDeviceListSyncService.list(null);
        //能耗仪表清单
        List<XaMeterList> xaMeterList = this.iXaMeterListService.list(null);
        xaDeviceList.forEach(dl -> {
            xaDeviceNumMap.put(dl.getDeviceNum(), dl.getWebsocketGroupId());
        });
        xaMeterList.forEach(ml ->{
            xaMeterNumMap.put(ml.getMeterNum(),9);
        });
        String groupId = "shenyang_real_time_" + System.currentTimeMillis();
        System.out.println("kafka消费者组号：" + groupId + "*********");
        System.setProperty("groupId", groupId);
    }

    @KafkaListener(topics = {"topic_kfk_iotproduction_xazz_1129"}, groupId = "${groupId}",containerFactory = "kafkaListenerContainerFactory")
    public void handleMessage2(ConsumerRecord<String, String> record) {
        // 当前时间
        Date currentTime = new Date();
        //消息时间戳
        long messageTimestamp = record.timestamp();
        // 设备实时数据
        XaIotDeviceRealInfo xaIotDeviceRealInfo = new XaIotDeviceRealInfo();
        //仪表实时数据
        XaMeterRealInfo xaMeterRealInfo = new XaMeterRealInfo();
        if (record.value() != null) {
            JSONObject jsonObject = JSON.parseObject(record.value());
            if (jsonObject != null) {
                // 设备编号
                String deviceNum = jsonObject.getString("__assetId__");
                if (deviceTimestamps.containsKey(deviceNum)) {
                    long previousTimestamp = deviceTimestamps.get(deviceNum);
                    if (messageTimestamp <= previousTimestamp) {
                        return ;
                    }
                }
                deviceTimestamps.put(deviceNum, messageTimestamp);
                if (xaDeviceNumMap.containsKey(deviceNum)) {
                    //设备编号
                    xaIotDeviceRealInfo.setDeviceNum(deviceNum);
                    //kafka主题
                    xaIotDeviceRealInfo.setTopic(record.topic());
                    //设备状态
                    Integer status = jsonObject.getInteger("Status");
                    xaIotDeviceRealInfo.setDeviceStatus(status);
                    //设备告警状态
                    xaIotDeviceRealInfo.setAlarm(jsonObject.containsKey("Alarm") ? jsonObject.getInteger("Alarm") : null);
                    //设备告警信息
                    xaIotDeviceRealInfo.setAlarmMsg(jsonObject.containsKey("AlarmMsg") ? jsonObject.getString("AlarmMsg") : null);
                    //设备时间
                    xaIotDeviceRealInfo.setDeviceTime(DateUtil.date(messageTimestamp));
                    //设备时间戳
                    Date date = new Date(messageTimestamp);
                    xaIotDeviceRealInfo.setDeviceTimeStr(DateUtil.format(date, "yyyy-MM-dd HH:mm:ss SSS"));
                    xaIotDeviceRealInfo.setDeviceTimestamp(DateUtil.format(date, "yyyy-MM-dd HH:mm:ss SSS"));
                    //设备分区
                    xaIotDeviceRealInfo.setPart(String.valueOf(record.partition()));
                    //设备点位
                    Map<String, Object> positionMap = new HashMap<>();
                    if (jsonObject.getString("__thingName__").contains("AGV")) {
                        positionMap.put("AGVPoseX", jsonObject.containsKey("AGVPoseX") ? jsonObject.getDouble("AGVPoseX") : null);
                        positionMap.put("AGVPoseY", jsonObject.containsKey("AGVPoseY") ? jsonObject.getDouble("AGVPoseY") : null);
                        positionMap.put("AGVPoseTheta", jsonObject.containsKey("AGVPoseTheta") ? jsonObject.getDouble("AGVPoseTheta") : null);
                        //电流
                        positionMap.put("BatCurrent", jsonObject.containsKey("BatCurrent") ? jsonObject.getDouble("BatCurrent") : null);
                        //电压
                        positionMap.put("BatVoltage", jsonObject.containsKey("BatVoltage") ? jsonObject.getDouble("BatVoltage") : null);
                        //功能状态
                        positionMap.put("BatPowerSupplyStatus", jsonObject.containsKey("BatPowerSupplyStatus") ? jsonObject.getDouble("BatPowerSupplyStatus") : null);
                        //电池电量
                        positionMap.put("BatteryCapacity", jsonObject.containsKey("BatteryCapacity") ? jsonObject.getDouble("BatteryCapacity") : null);
                        //总行驶里程
                        positionMap.put("TotalMileage", jsonObject.containsKey("TotalMileage") ? jsonObject.getDouble("TotalMileage") : null);
                        //日行驶里程
                        positionMap.put("DayMileage", jsonObject.containsKey("DayMileage") ? jsonObject.getDouble("DayMileage") : null);
                        //工作任务状态
                        positionMap.put("TaskStatus", jsonObject.containsKey("TaskStatus") ? jsonObject.getDouble("TaskStatus") : null);
                    } else {
                        String[] keys = {"J1", "J2", "J3", "J4", "J5", "J6", "E1","J7","J8","J9","FixName","WeldDetect"};
                        for (String key : keys) {
                            if (jsonObject.containsKey(key)) {
                                positionMap.put(key, jsonObject.get(key));
                            }
                        }
                    }
                    String position = JSON.toJSONString(positionMap);
                    xaIotDeviceRealInfo.setPointPosition(position);
                    //设备websocket分组
                    xaIotDeviceRealInfo.setWebsocketGroupId(xaDeviceNumMap.get(deviceNum));
                    xaIotDeviceRealInfo.setGmtModified(currentTime);
                    xaIotDeviceRealInfo.setGmtCreated(currentTime);
                }else if(xaMeterNumMap.containsKey(deviceNum)){
                    xaMeterRealInfo.setMeterNum(deviceNum);
                    xaMeterRealInfo.setSabc(jsonObject.containsKey("Sabc") ? jsonObject.getFloat("Sabc") : null);
                    xaMeterRealInfo.setEpp(jsonObject.containsKey("EPP") ? jsonObject.getFloat("EPP") : null);
                    xaMeterRealInfo.setPart(String.valueOf(record.partition()));
                    xaMeterRealInfo.setTopic(record.topic());
                    xaMeterRealInfo.setMeterTime(DateUtil.date(messageTimestamp));
                    xaMeterRealInfo.setWebsocketGroupId(xaMeterNumMap.get(deviceNum));
                }
            }
        }
        if (xaIotDeviceRealInfo.getDeviceNum() != null && !xaIotDeviceRealInfo.getDeviceNum().isEmpty()) {
            String deviceInfo = JSON.toJSONString(xaIotDeviceRealInfo);
            redisUtils.set("XaKafka:" + xaIotDeviceRealInfo.getDeviceNum() + ":" + xaIotDeviceRealInfo.getPart(), deviceInfo);
        }else if(xaMeterRealInfo.getMeterNum() != null && !xaMeterRealInfo.getMeterNum().isEmpty()){
            String meterInfo = JSON.toJSONString(xaMeterRealInfo);
            redisUtils.set("XaMeterKafka:" + xaMeterRealInfo.getMeterNum() + ":" + xaMeterRealInfo.getPart(), meterInfo);
        }
    }
}
